@@ -2,6 +2,7 @@ package HTTP::Server::PSGI;
 use strict;
 use warnings;
 
+use Carp ();
 use Plack;
 use Plack::HTTPParser qw( parse_http_request );
 use IO::Socket::INET;
@@ -37,27 +38,16 @@ sub new {
         host               => $args{host} || 0,
         port               => $args{port} || 8080,
         timeout            => $args{timeout} || 300,
-        keepalive_timeout  => $args{keepalive_timeout} || 2,
-        max_keepalive_reqs => $args{max_keepalive_reqs},
         server_software    => $args{server_software} || $class,
         server_ready       => $args{server_ready} || sub {},
-        max_workers        => $args{max_workers} || 1,
         max_reqs_per_child => $args{max_reqs_per_child} || 100,
     }, $class;
 
-    if ($self->{max_workers} > 1) {
-        try {
-            require Parallel::Prefork;
-            $self->{prefork} = 1;
-            $self->{max_keepalive_reqs} ||= 100;
-            $self->{server_software} .= " (prefork)";
-        } catch {
-            die "You need to install Parallel::Prefork to run multi workers (max_workers=$self->{max_workers}): $_";
-        };
-    }
-
-    unless ($self->{prefork}) {
-        $self->{max_keepalive_reqs} ||= 1;
+    if ($args{max_workers} && $args{max_workers} > 1) {
+        Carp::carp(
+            "Preforking in $class is deprecated. Falling back to the non-forking mode. ",
+            "If you need preforking, use Starman or Starlet instead and run like `plackup -s Starlet`",
+        );
     }
 
     $self;
@@ -66,12 +56,7 @@ sub new {
 sub run {
     my($self, $app) = @_;
     $self->setup_listener();
-
-    if ($self->{prefork}) {
-        $self->run_prefork($app);
-    } else {
-        $self->accept_loop($app);
-    }
+    $self->accept_loop($app);
 }
 
 sub setup_listener {
@@ -87,24 +72,6 @@ sub setup_listener {
     $self->{server_ready}->($self);
 }
 
-sub run_prefork {
-    my($self, $app) = @_;
-
-    my $pm = Parallel::Prefork->new({
-        max_workers => $self->{max_workers},
-        trap_signals => {
-            TERM => 'TERM',
-            HUP  => 'TERM',
-        },
-    });
-    while ($pm->signal_received ne 'TERM') {
-        $pm->start and next;
-        $self->accept_loop($app, $self->{max_reqs_per_child});
-        $pm->finish;
-    }
-    $pm->wait_all_children;
-}
-
 sub accept_loop {
     # TODO handle $max_reqs_per_child
     my($self, $app, $max_reqs_per_child) = @_;
@@ -117,43 +84,32 @@ sub accept_loop {
         if (my $conn = $self->{listen_sock}->accept) {
             $conn->setsockopt(IPPROTO_TCP, TCP_NODELAY, 1)
                 or die "setsockopt(TCP_NODELAY) failed:$!";
-            my $req_count = 0;
-            while (1) {
-                ++$req_count;
-                ++$proc_req_count;
-                my $env = {
-                    SERVER_PORT => $self->{port},
-                    SERVER_NAME => $self->{host},
-                    SCRIPT_NAME => '',
-                    REMOTE_ADDR => $conn->peerhost,
-                    'psgi.version' => [ 1, 1 ],
-                    'psgi.errors'  => *STDERR,
-                    'psgi.url_scheme' => 'http',
-                    'psgi.run_once'     => Plack::Util::FALSE,
-                    'psgi.multithread'  => Plack::Util::FALSE,
-                    'psgi.multiprocess' => $self->{prefork},
-                    'psgi.streaming'    => Plack::Util::TRUE,
-                    'psgi.nonblocking'  => Plack::Util::FALSE,
-                    'psgix.input.buffered' => Plack::Util::TRUE,
-                    'psgix.io'          => $conn,
-                };
+            ++$proc_req_count;
+            my $env = {
+                SERVER_PORT => $self->{port},
+                SERVER_NAME => $self->{host},
+                SCRIPT_NAME => '',
+                REMOTE_ADDR => $conn->peerhost,
+                'psgi.version' => [ 1, 1 ],
+                'psgi.errors'  => *STDERR,
+                'psgi.url_scheme' => 'http',
+                'psgi.run_once'     => Plack::Util::FALSE,
+                'psgi.multithread'  => Plack::Util::FALSE,
+                'psgi.multiprocess' => Plack::Util::FALSE,
+                'psgi.streaming'    => Plack::Util::TRUE,
+                'psgi.nonblocking'  => Plack::Util::FALSE,
+                'psgix.input.buffered' => Plack::Util::TRUE,
+                'psgix.io'          => $conn,
+            };
 
-                # no need to take care of pipelining since this module is a HTTP/1.0 server
-                my $may_keepalive = $req_count < $self->{max_keepalive_reqs};
-                if ($may_keepalive && $max_reqs_per_child && $proc_req_count >= $max_reqs_per_child) {
-                    $may_keepalive = undef;
-                }
-                $self->handle_connection($env, $conn, $app, $may_keepalive, $req_count != 1)
-                    or last;
-                # TODO add special cases for clients with broken keep-alive support, as well as disabling keep-alive for HTTP/1.0 proxies
-            }
+            $self->handle_connection($env, $conn, $app);
             $conn->close;
         }
     }
 }
 
 sub handle_connection {
-    my($self, $env, $conn, $app, $use_keepalive, $is_keepalive) = @_;
+    my($self, $env, $conn, $app) = @_;
 
     my $buf = '';
     my $res = [ 400, [ 'Content-Type' => 'text/plain' ], [ 'Bad Request' ] ];
@@ -161,19 +117,10 @@ sub handle_connection {
     while (1) {
         my $rlen = $self->read_timeout(
             $conn, \$buf, MAX_REQUEST_SIZE - length($buf), length($buf),
-            $is_keepalive ? $self->{keepalive_timeout} : $self->{timeout},
+            $self->{timeout},
         ) or return;
         my $reqlen = parse_http_request($buf, $env);
         if ($reqlen >= 0) {
-            # handle request
-            if ($use_keepalive) {
-                if (my $c = $env->{HTTP_CONNECTION}) {
-                    $use_keepalive = undef
-                        unless $c =~ /^\s*keep-alive\s*/i;
-                } else {
-                    $use_keepalive = undef;
-                }
-            }
             $buf = substr $buf, $reqlen;
             if (my $cl = $env->{CONTENT_LENGTH}) {
                 my $buffer = Plack::TempBuffer->new($cl);
@@ -206,20 +153,20 @@ sub handle_connection {
     }
 
     if (ref $res eq 'ARRAY') {
-        $self->_handle_response($res, $conn, \$use_keepalive);
+        $self->_handle_response($res, $conn);
     } elsif (ref $res eq 'CODE') {
         $res->(sub {
-            $self->_handle_response($_[0], $conn, \$use_keepalive);
+            $self->_handle_response($_[0], $conn);
         });
     } else {
         die "Bad response $res";
     }
 
-    return $use_keepalive;
+    return;
 }
 
 sub _handle_response {
-    my($self, $res, $conn, $use_keepalive_r) = @_;
+    my($self, $res, $conn) = @_;
 
     my @lines = (
         "Date: @{[HTTP::Date::time2str()]}\015\012",
@@ -228,19 +175,9 @@ sub _handle_response {
 
     Plack::Util::header_iter($res->[1], sub {
         my ($k, $v) = @_;
-        if (lc $k eq 'connection') {
-            $$use_keepalive_r = undef
-                if $$use_keepalive_r && lc $v ne 'keep-alive';
-        } else {
-            push @lines, "$k: $v\015\012";
-        }
+        push @lines, "$k: $v\015\012";
     });
-    if ($$use_keepalive_r) {
-        $$use_keepalive_r = undef
-            unless Plack::Util::header_exists($res->[1], 'Content-Length');
-    }
-    push @lines, "Connection: keep-alive\015\012"
-        if $$use_keepalive_r;
+
     unshift @lines, "HTTP/1.0 $res->[0] @{[ HTTP::Status::status_message($res->[0]) ]}\015\012";
     push @lines, "\015\012";
 
@@ -348,8 +285,7 @@ HTTP::Server::PSGI - Standalone PSGI compatible HTTP server
 =head1 DESCRIPTION
 
 HTTP::Server::PSGI is a standalone, single-process and PSGI compatible
-HTTP server implementations. It runs reasonably fast and HTTP/1.0 and
-Keep-Alive requests are supported.
+HTTP server implementations.
 
 This server should be great for the development and testig, but might
 not be suitable for production.
@@ -357,8 +293,10 @@ not be suitable for production.
 Some features in HTTP/1.1, notably chunked requests, responses and
 pipeline requests are B<NOT> supported yet.
 
-See L<Starman> if you want a multi-process prefork server with some
-HTTP/1.1 features support.
+=head1 PREFORKING
+
+L<HTTP::Server::PSGI> does B<NOT> support preforking. See L<Starman>
+or L<Starlet> if you want a multi-process prefork web servers.
 
 =head1 AUTHOR
 
@@ -368,6 +306,6 @@ Tatsuhiko Miyagawa
 
 =head1 SEE ALSO
 
-L<Plack::Handler::Standalone>
+L<Plack::Handler::Standalone> L<Starman> L<Starlet>
 
 =cut
